import typing
import time
import urllib.parse
import requests
import os
import datetime
import dateutil.parser
import lxml
import lxml.etree
import presalytics
import presalytics.client.api
import presalytics.story.components
import presalytics.client
import presalytics.lib.exceptions
import presalytics.story.outline
import presalytics.lib.util
import presalytics.lib.plugins.ooxml
if typing.TYPE_CHECKING:
    from presalytics.story.outline import StoryOutline, Page, Widget
    from presalytics.client.presalytics_story import Story as ApiStory


class OoxmlEndpointMap(object):
    BASE_URL = "https://api.presalytics.io/ooxml-automation"

    CHART = "Charts"
    CONNECTION_SHAPE = "ConnectionShapes"
    DOCUMENT = "Documents"
    GROUP = "Groups"
    IMAGE = "Images"
    SHAPE = "Shapes"
    SHAPETREE = "ShapeTrees"
    SLIDE = "Slides"
    TABLE = "Tables"
    THEME = "Themes"


    def _build_object_type_map(self):
        return {
            "Chart": [
                OoxmlEndpointMap.CHART,
            ],
            "Slide": [
                OoxmlEndpointMap.GROUP,
                OoxmlEndpointMap.SHAPE,
                OoxmlEndpointMap.SHAPETREE, 
                OoxmlEndpointMap.CONNECTION_SHAPE,
                OoxmlEndpointMap.SLIDE
            ],
            "Table": [
                OoxmlEndpointMap.TABLE
            ],
            "Theme": [
                OoxmlEndpointMap.THEME
            ],
            "Shared": [
                OoxmlEndpointMap.IMAGE
            ],
            "EMPTY": [
                OoxmlEndpointMap.DOCUMENT
            ]
        }
    
    

    def __init__(self, endpoint, baseurl: str = None):
        if endpoint not in OoxmlEndpointMap.__dict__.values():
            raise presalytics.lib.exceptions.ValidationError("{0} is not a valid endpoint ID".format(endpoint))
        self.endpoint_id = endpoint
        if not baseurl:
            custom_hosts = presalytics.CONFIG.get("HOSTS", None)
            if custom_hosts:
                ooxml_host = custom_hosts.get("OOXML_AUTOMATION", None)
                if ooxml_host:
                    self.baseurl = ooxml_host
            if not self.baseurl:
                self.baseurl = OoxmlEndpointMap.BASE_URL
        else:
            self.baseurl = baseurl
        self.root_url = urllib.parse.urljoin(self.baseurl, self.endpoint_id)
        self.OBJECT_TYPE_MAP = self._build_object_type_map()
    
    def get_object_type(self):
        for key, val in self.OBJECT_TYPE_MAP.items():
            for test_ep in val:
                if test_ep == self.endpoint_id:
                    if key == "EMPTY":
                        return self.endpoint_id
                    else:
                        return "{0}.{1}".format(key, self.endpoint_id)
        message = "Invalid EndpointMap configuration: {0} is not in OBJECT_TYPE_MAP".format(self.endpoint_id)
        raise presalytics.lib.exceptions.ValidationError(message)

    def get_id_url(self, id):
        return os.path.join(self.root_url, id)

    def get_svg_url(self, id):
        return os.path.join(self.root_url, "Svg", id)

    def get_xml_url(self, id):
        return os.path.join(self.root_url, "OpenOfficeXml", id)

    @classmethod
    def connection_shape(cls, baseurl=None):
        return cls(OoxmlEndpointMap.CONNECTION_SHAPE, baseurl)

    @classmethod
    def chart(cls, baseurl=None):
        return cls(OoxmlEndpointMap.CHART, baseurl)

    @classmethod
    def document(cls, baseurl=None):
        return cls(OoxmlEndpointMap.DOCUMENT, baseurl)

    @classmethod
    def image(cls, baseurl=None):
        return cls(OoxmlEndpointMap.IMAGE, baseurl)

    @classmethod
    def shape(cls, baseurl=None):
        return cls(OoxmlEndpointMap.SHAPE, baseurl)

    @classmethod
    def shapetree(cls, baseurl=None):
        return cls(OoxmlEndpointMap.SHAPETREE, baseurl)

    @classmethod
    def slide(cls, baseurl=None):
        return cls(OoxmlEndpointMap.SLIDE, baseurl)

    @classmethod
    def table(cls, baseurl=None):
        return cls(OoxmlEndpointMap.table, baseurl)

    @classmethod
    def chart(cls, baseurl=None):
        return cls(OoxmlEndpointMap.THEME, baseurl)






    



class OoxmlWidgetBase(presalytics.story.components.WidgetBase):
    endpoint_map: OoxmlEndpointMap
    data: typing.Dict

    __plugins__ = [
        {
            'name': 'external_scripts',
            'kind': 'script',
            'config': {
                'approved_scripts_key': 'ooxml'
            }
        }
    ]

    def to_html(self, **kwargs):
        return self.svg_html

    def get_svg(self, id, timeout_iterator=0) -> str:
        svg_url = self.endpoint_map.get_svg_url(id)
        client = presalytics.client.api.Client()
        auth_header = client.get_auth_header()
        response = requests.get(svg_url, headers=auth_header)
        svg_data = response.text
        if response.status_code != 200:
            raise presalytics.lib.exceptions.ApiException(default_exception=response.content)
        if response.text.startswith("Temp data"):
            if timeout_iterator > 5:
                raise presalytics.lib.exceptions.ApiException()
            else:
                time.sleep(2)
                svg_data = self.get_svg(id, timeout_iterator)
        return svg_data


class OoxmlFileWidget(OoxmlWidgetBase):
    object_name: typing.Optional[str]
    ooxml_id: str
    file_last_modified: datetime.datetime
    previous_ooxml_version: typing.Dict[str, str]

    __component_kind__ = 'ooxml-file-object'

    def __init__(self,
                 filename,
                 name=None,
                 endpoint_map=None,
                 object_name=None,
                 previous_ooxml_version={},
                 file_last_modified=None,
                 document_ooxml_id=None,
                 story_id=None,
                 object_ooxml_id=None):
        self.filename = os.path.basename(filename)
        if endpoint_map:
            self.endpoint_map = endpoint_map
        else:
            if filename.split(".")[-1] in ["pptx", "ppt"]:
                self.endpoint_map = OoxmlEndpointMap(OoxmlEndpointMap.SLIDE)
        if object_name:
            self.object_name = object_name
        else:
            self.object_name = None
        if name:
            self.name = name
        else:
            if self.object_name:
                self.name = self.object_name
            else:
                self.name = filename
        self.previous_ooxml_version = previous_ooxml_version
        if file_last_modified:
            self.file_last_modified = file_last_modified
        else:
            self.file_last_modified = datetime.datetime.utcnow()
        self.document_ooxml_id = document_ooxml_id
        self.object_ooxml_id = object_ooxml_id
        self.story_id = story_id
        self.update()
        self.svg_html = self.get_svg(self.object_ooxml_id)

    def update(self):
        """
        If the file is available locally, this renders that updated file and pushes
        the updated rendering data to the server
        """
        story: 'ApiStory'
        page: 'Page'
        widget: 'Widget'

        search_paths = list(set(presalytics.autodiscover_paths))
        if os.getcwd() not in search_paths:
            search_paths.append(os.getcwd())
        for path in search_paths:
            fpath = os.path.join(path, self.filename)
            if os.path.exists(fpath):
                # update only the file has been modified sine last time
                this_file_last_modified = datetime.datetime.utcfromtimestamp(os.path.getmtime(fpath))
                if self.file_last_modified is None or self.file_last_modified <= this_file_last_modified:
                    client = presalytics.client.api.Client()
                    story, status, headers = client.story.story_id_file_post_with_http_info(self.story_id, file=fpath, replace_existing=True, obsolete_id=self.document_ooxml_id)
                    if status >= 299:
                        raise presalytics.lib.exceptions.ApiError()
                    self.previous_ooxml_version = {
                        "document_ooxml_id": self.document_ooxml_id,
                        "object_ooxml_id": self.object_ooxml_id
                    }
                    new_outline = presalytics.story.outline.StoryOutline.load(story.outline)
                    for page in new_outline.pages:
                        found = False
                        for widget in page.widgets:
                            if widget.name == self.object_name:
                                self.document_ooxml_id = widget.data["document_ooxml_id"]
                                found = True
                                break
                        if found:
                            break
                    if not found:
                        message = "Unable to find widget object name {0} in new story outline.  Has this widget been deleted?".format(self.object_name)
                        raise presalytics.lib.exceptions.ValidationError(message)

                    # Get object tree, compare to object name
                    child_tree = client.ooxml_automation.documents_childobjects_get_id(self.document_ooxml_id)
                    target_dto = None

                    try:
                        target_dto = next(x for x in child_tree if x.entity_name == self.object_name)
                    except StopIteration:
                        pass
                    # if name not in object or _object_name is none, get first item of type in end_point id
                    if not target_dto:
                        try:
                            target_dto = next(x for x in child_tree if x.object_type.split(".")[1] == self.endpoint_map.endpoint_id)
                        except StopIteration:
                            message = "Child tree of document {0} does not have a child object of type {1} or name {2}.".format(self.ooxml_id, self.endpoint_map.endpoint_id, self.object_name)
                            raise presalytics.lib.exceptions.InvalidConfigurationError(message)
                    # set widget parameters for recreation server-side (without file)
                    self.object_ooxml_id = target_dto.entity_id
                    self.file_last_modified = presalytics.lib.util.roundup_date_modified(this_file_last_modified)


    @classmethod
    def deserialize(cls, component, **kwargs):
        init_args = {
            "filename": component.data["filename"],
            "endpoint_map": OoxmlEndpointMap(component.data["endpoint_id"]),
            "object_name": component.data["object_name"],
            "name": component.name,
        }
        if "document_ooxml_id" in component.data:
            init_args.update(
                {
                    "document_ooxml_id": component.data["document_ooxml_id"]
                }
            )
            if "object_ooxml_id" in component.data:
                init_args.update(
                    {
                        "object_ooxml_id": component.data["object_ooxml_id"]
                    }
                )
            if "file_last_modified" in component.data:
                init_args.update(
                    {
                        "file_last_modified": dateutil.parser.parse(component.data["file_last_modified"])
                    }
                )
            if "previous_ooxml_version" in component.data:
                init_args.update(
                    {
                        "previous_ooxml_version": component.data["previous_ooxml_version"]
                    }
                )
            if "story_id" in component.data:
                init_args.update(
                    {
                        "story_id": component.data["story_id"]
                    }
                )
            return cls(**init_args)

    def serialize(self):
        self.update()
        data = {
            "filename": self.filename,
            "object_name": self.object_name,
            "endpoint_id": self.endpoint_map.endpoint_id,
            "document_ooxml_id": self.document_ooxml_id,
            "object_ooxml_id": self.object_ooxml_id,
            "story_id": self.story_id
        }
        if self.file_last_modified:
            data.update(
                {
                    "file_last_modified": self.file_last_modified.isoformat()
                }
            )
        if self.previous_ooxml_version:
            data.update(
                {
                    "previous_ooxml_version": self.previous_ooxml_version
                }
            )
        widget = presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            data=data,
            plugins=None
        )
        return widget



"""
Called at runtime by BackgrounderMixIn

This is WIP
"""
import sys
import pickle
import typing
import time
import presalytics.lib.widgets.backgrounder

backgrounder_instance: presalytics.lib.widgets.backgrounder.BackgrounderMixIn = None
process_identifier = None
sleep_interval = 300
keep_open = True
start_time = time.time()

while keep_open is True:
    new_data = sys.stdin.readlines()
    obj = pickle.load(new_data)
    if isinstance(obj, presalytics.lib.widgets.backgrounder.BackgrounderMixIn):
        backgrounder_instance = obj
    if isinstance(obj, dict): 
        if obj.get("keep_open", None):
            keep_open = obj.get("keep_open")
        if obj.get("process_identifer"

    if backgrounder_instance and process_identifier:
        backgrounder_instance.execute()
    else:
        runtime = time.time - start_time
        if runtime > sleep_interval:
            keep_open = False

        


        



import abc
import typing
import logging
import collections
import lxml
import lxml.etree
import presalytics
import presalytics.lib.exceptions
import presalytics.lib.registry


logger = logging.getLogger('presalytics.lib.plugins.base')


class PluginBase(abc.ABC):
    """
    A plugin converts a dictionary of configuration values into an html script.
    Typically plugins are are used as reable mapping classes to add script tags to
    to a rendered html body.

    Attributes:
    ----------

    __plugin_kind__: str
        The __plugin_kind__ is a static string that uniquely identifies this plugin to classes
        the render story outlines (e.g., presaltyics.story.revealer.Revealer).

    """
    __plugin_kind__: str
    __plugin_name__: str
    __dependencies__: typing.List[typing.Dict]

    __dependencies__ = []

    config: typing.Dict

    def __init__(self, **kwargs):
        self.validate_metadata()

    def validate_metadata(self):
        if self.__plugin_kind__ == "" or self.__plugin_name__ == "":
            message = "Plugin class {0} has not defined either __plugin_kind__ or __plugin_name__ metadata".format(self.__class__.__name__)
            raise presalytics.lib.exceptions.ValidationError(message)

    @abc.abstractmethod
    def get_tag(self, config: typing.Dict[str, typing.Any], **kwargs) -> str:
        """
        Generic method to be implemented by all inheriting classes. Allow plugin
        to be render without knowledge of the underlying plugin kind.

        Parameters:
        ----------

        config: Dict
            A set configuration values for the plugin.  Required keys should be
            specified by the docstring of the inheriting classes.
            Can be implemented via a mypy_extensions.TypedDict object if warranted.
            The presalytics.story.outline.Plugin's config element should be passed
            as this value to render the script.

        Returns:
        ----------

        A string carrying a html script that can be embedded in a html document

        """
        raise NotImplementedError


class ScriptPlugin(PluginBase):
    """
    A script plugin render configures templated scripts to append to the body
    tag of an html document
    """
    __plugin_kind__ = 'script'

    def get_tag(self, config, **kwargs):
        return self.to_script(config, **kwargs)

    @abc.abstractmethod
    def to_script(self, config: typing.Dict[str, typing.Any], **kwargs) -> str:
        """
        The to_script method maps a dictionary to an html script tag.

        Parameters:
        ----------

        config: Dict
            A set configuration values for the plugin.  Required keys should be
            specified by the docstring of the inheriting classes.
            Can be implemented via a mypy_extensions.TypedDict object if warranted.
            The presalytics.story.outline.Plugin's config element should be passed
            as this value to render the script.

        Returns:
        ----------

        A string carrying a html script that can be embedded in a html document

        """
        raise NotImplementedError


class StylePlugin(PluginBase):
    """
    A script plugin render configures templated scripts to append to the body
    tag of an html document
    """
    __plugin_kind__ = 'style'

    def get_tag(self, config, **kwargs):
        return self.to_style(config, **kwargs)

    @abc.abstractmethod
    def to_style(self, config: typing.Dict[str, typing.Any], **kwargs) -> str:
        """
        The to_style method maps a dictionary to an html style tag.

        Parameters:
        ----------

        config: Dict
            A set configuration values for the plugin.  Required keys should be
            specified by the docstring of the inheriting classes.
            Can be implemented via a mypy_extensions.TypedDict object if warranted.
            The presalytics.story.outline.Plugin's config element should be passed
            as this value to render the script.

        Returns:
        ----------

        A string carrying a html script that can be embedded in a html document

        """
        raise NotImplementedError


class PluginRegistry(presalytics.lib.registry.RegistryBase):
    """
    The Plugin Registry class
    """
    def __init__(self, **kwargs):
        super(PluginRegistry, self).__init__(**kwargs)

    def get_name(self, klass):
        return getattr(klass, "__plugin_name__", None)

    def get_type(self, klass):
        return getattr(klass, "__plugin_kind__", None)


class Graph():
    def __init__(self, vertices):
        self.graph = collections.defaultdict(list)
        self.V = vertices

    def addEdge(self, u, v):
        self.graph[u].append(v)

    def isCyclicUtil(self, v, visited, recStack):

        # Mark current node as visited and
        # adds to recursion stack
        visited[v] = True
        recStack[v] = True

        # Recur for all neighbours
        # if any neighbour is visited and in
        # recStack then graph is cyclic
        for neighbour in self.graph[v]:
            if not visited[neighbour]:
                if self.isCyclicUtil(neighbour, visited, recStack):
                    return True
            elif recStack[neighbour] is True:
                return True

        # The node needs to be poped from
        # recursion stack before function ends
        recStack[v] = False
        return False

    # Returns true if graph is cyclic else false
    def isCyclic(self):
        visited = [False] * self.V
        recStack = [False] * self.V
        for node in range(self.V):
            if not visited[node]:
                if self.isCyclicUtil(node, visited, recStack):
                    return True
        return False


class PluginManager(object):

    dependency_map: typing.Dict[str, typing.Dict[str, typing.Any]]
    dependency_order: typing.List[str]

    def __init__(self, plugins: typing.List[typing.Dict], ignore_errors=True):
        self.ignore_errors = ignore_errors
        self.dependency_map = {}
        self.dependency_order = []
        self.load_dependencies(plugins)
        self.dependency_graph = Graph(len(plugins))
        self.load_graph()
        self.check_for_cyclic_dependencies()
        self.sort_plugins()

    def sort_plugins(self):
        resort = False
        for key in self.dependency_order:
            current_index = self.dependency_order.index(key)
            lowest_dep = None
            deps = self.dependency_map[key]["dependencies"]
            for dep in deps:
                for dep_key, dep_map in self.dependency_map.items():
                    if dep == dep_map["plugin"]:
                        dep_index = self.dependency_order.index(dep_key)
                        if lowest_dep:
                            if dep_index < lowest_dep:
                                lowest_dep = dep_index
                        else:
                            lowest_dep = dep_index
                        break
            if lowest_dep:
                if lowest_dep > current_index:
                    self.dependency_order.insert(lowest_dep + 1, self.dependency_order.pop(current_index))
                    resort = True
                    break
        if resort:
            self.sort_plugins()

    def add_plugin_to_dep_map(self, plugin):
        current_plugins = [x["plugin"] for x in self.dependency_map.values()]
        if plugin not in current_plugins:
            lookup_key = plugin["kind"] + "." + plugin["name"]
            plugin_class = presalytics.PLUGINS.get(lookup_key)
            if plugin_class is None:
                message = "Required plugin {0} not found.".format(lookup_key)
                if self.ignore_errors:
                    logger.error(message)
                else:
                    raise presalytics.lib.exceptions.ValidationError(message)
            else:
                index = len(self.dependency_map.keys())
                dict_key = str(index) + "." + lookup_key
                self.dependency_order.append(dict_key)
                entry = {
                    dict_key: {
                        "plugin": plugin,
                        "class": plugin_class,
                        "dependencies": plugin_class.__dependencies__
                    }
                }
                self.dependency_map.update(entry)
                if len(plugin_class.__dependencies__) > 0:
                    self.load_dependencies(plugin_class.__dependencies__)

    def load_dependencies(self, plugins):
        for plugin in plugins:
            self.add_plugin_to_dep_map(plugin)

    def load_graph(self):
        for key, val in self.dependency_map.items():
            for dep in val["dependencies"]:
                key_node = list(self.dependency_map.keys()).index(key)
                plugin_list = [x["plugin"] for x in self.dependency_map.values()]
                dep_node = plugin_list.index(dep)
                self.dependency_graph.addEdge(key_node, dep_node)

    def check_for_cyclic_dependencies(self) -> None:
        if self.dependency_graph.isCyclic():
            message = "Loaded plugins container circular dependencies.  Plugins may not load in correct order"
            if self.ignore_errors:
                logger.error(message)
            else:
                raise presalytics.lib.exceptions.ValidationError(message)

    def get_styles(self) -> typing.List[str]:
        return self.render_plugins('style')

    def get_scripts(self) -> typing.List[str]:
        return self.render_plugins('script')

    def render_plugins(self, plugin_kind: str) -> typing.List[str]:
        rendered_list = []
        for key in self.dependency_order:
            dep_map = self.dependency_map[key]
            if dep_map["plugin"]["kind"] == plugin_kind:
                plugin_config = dep_map["plugin"]
                plugin_class = dep_map["class"]
                plugin_instance = plugin_class()
                tag = plugin_instance.get_tag(config=plugin_config["config"])
                rendered_list.append(tag)
        return rendered_list

    @staticmethod
    def get_plugins_from_nested_dict(source_dict: typing.Dict, plugin_list: typing.List[typing.Dict] = []) -> typing.List[typing.Dict]:
        for key, val in source_dict.items():
            if key == "plugins":
                if isinstance(val, list):
                    for list_item in val:
                        if isinstance(list_item, dict):
                            if "config" in list_item and "name" in list_item and "kind" in list_item:
                                plugin_list.append(list_item)
            if isinstance(val, dict):
                PluginManager.get_plugins_from_nested_dict(val, plugin_list)
            if isinstance(val, list):
                for list_item in val:
                    if isinstance(list_item, dict):
                        PluginManager.get_plugins_from_nested_dict(list_item, plugin_list)
        return plugin_list



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class ChartChartDataDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'chart_id': 'str',
        'chart': 'object',
        'row_collection': 'object',
        'column_collection': 'object',
        'data_points': 'object',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'chart_id': 'chartId',
        'chart': 'chart',
        'row_collection': 'rowCollection',
        'column_collection': 'columnCollection',
        'data_points': 'dataPoints',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, chart_id=None, chart=None, row_collection=None, column_collection=None, data_points=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None, local_vars_configuration=None):  # noqa: E501
        """ChartChartDataDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._chart_id = None
        self._chart = None
        self._row_collection = None
        self._column_collection = None
        self._data_points = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.chart_id = chart_id
        if chart is not None:
            self.chart = chart
        if row_collection is not None:
            self.row_collection = row_collection
        if column_collection is not None:
            self.column_collection = column_collection
        self.data_points = data_points
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def chart_id(self):
        """Gets the chart_id of this ChartChartDataDetails.  # noqa: E501


        :return: The chart_id of this ChartChartDataDetails.  # noqa: E501
        :rtype: str
        """
        return self._chart_id

    @chart_id.setter
    def chart_id(self, chart_id):
        """Sets the chart_id of this ChartChartDataDetails.


        :param chart_id: The chart_id of this ChartChartDataDetails.  # noqa: E501
        :type: str
        """

        self._chart_id = chart_id

    @property
    def chart(self):
        """Gets the chart of this ChartChartDataDetails.  # noqa: E501


        :return: The chart of this ChartChartDataDetails.  # noqa: E501
        :rtype: object
        """
        return self._chart

    @chart.setter
    def chart(self, chart):
        """Sets the chart of this ChartChartDataDetails.


        :param chart: The chart of this ChartChartDataDetails.  # noqa: E501
        :type: object
        """

        self._chart = chart

    @property
    def row_collection(self):
        """Gets the row_collection of this ChartChartDataDetails.  # noqa: E501


        :return: The row_collection of this ChartChartDataDetails.  # noqa: E501
        :rtype: object
        """
        return self._row_collection

    @row_collection.setter
    def row_collection(self, row_collection):
        """Sets the row_collection of this ChartChartDataDetails.


        :param row_collection: The row_collection of this ChartChartDataDetails.  # noqa: E501
        :type: object
        """

        self._row_collection = row_collection

    @property
    def column_collection(self):
        """Gets the column_collection of this ChartChartDataDetails.  # noqa: E501


        :return: The column_collection of this ChartChartDataDetails.  # noqa: E501
        :rtype: object
        """
        return self._column_collection

    @column_collection.setter
    def column_collection(self, column_collection):
        """Sets the column_collection of this ChartChartDataDetails.


        :param column_collection: The column_collection of this ChartChartDataDetails.  # noqa: E501
        :type: object
        """

        self._column_collection = column_collection

    @property
    def data_points(self):
        """Gets the data_points of this ChartChartDataDetails.  # noqa: E501


        :return: The data_points of this ChartChartDataDetails.  # noqa: E501
        :rtype: object
        """
        return self._data_points

    @data_points.setter
    def data_points(self, data_points):
        """Sets the data_points of this ChartChartDataDetails.


        :param data_points: The data_points of this ChartChartDataDetails.  # noqa: E501
        :type: object
        """

        self._data_points = data_points

    @property
    def id(self):
        """Gets the id of this ChartChartDataDetails.  # noqa: E501


        :return: The id of this ChartChartDataDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ChartChartDataDetails.


        :param id: The id of this ChartChartDataDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this ChartChartDataDetails.  # noqa: E501


        :return: The date_created of this ChartChartDataDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this ChartChartDataDetails.


        :param date_created: The date_created of this ChartChartDataDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this ChartChartDataDetails.  # noqa: E501


        :return: The user_created of this ChartChartDataDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this ChartChartDataDetails.


        :param user_created: The user_created of this ChartChartDataDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this ChartChartDataDetails.  # noqa: E501


        :return: The date_modified of this ChartChartDataDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this ChartChartDataDetails.


        :param date_modified: The date_modified of this ChartChartDataDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this ChartChartDataDetails.  # noqa: E501


        :return: The user_modified of this ChartChartDataDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this ChartChartDataDetails.


        :param user_modified: The user_modified of this ChartChartDataDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ChartChartDataDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ChartChartDataDetails):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class ThemeColorsDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'theme_id': 'str',
        'theme': 'object',
        'accent1': 'str',
        'accent2': 'str',
        'accent3': 'str',
        'accent4': 'str',
        'accent5': 'str',
        'accent6': 'str',
        'light1': 'str',
        'light2': 'str',
        'dark1': 'str',
        'dark2': 'str',
        'hyperlink': 'str',
        'followed_hyperlink': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'name': 'name',
        'theme_id': 'themeId',
        'theme': 'theme',
        'accent1': 'accent1',
        'accent2': 'accent2',
        'accent3': 'accent3',
        'accent4': 'accent4',
        'accent5': 'accent5',
        'accent6': 'accent6',
        'light1': 'light1',
        'light2': 'light2',
        'dark1': 'dark1',
        'dark2': 'dark2',
        'hyperlink': 'hyperlink',
        'followed_hyperlink': 'followedHyperlink',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, name=None, theme_id=None, theme=None, accent1=None, accent2=None, accent3=None, accent4=None, accent5=None, accent6=None, light1=None, light2=None, dark1=None, dark2=None, hyperlink=None, followed_hyperlink=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None, local_vars_configuration=None):  # noqa: E501
        """ThemeColorsDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._theme_id = None
        self._theme = None
        self._accent1 = None
        self._accent2 = None
        self._accent3 = None
        self._accent4 = None
        self._accent5 = None
        self._accent6 = None
        self._light1 = None
        self._light2 = None
        self._dark1 = None
        self._dark2 = None
        self._hyperlink = None
        self._followed_hyperlink = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.name = name
        self.theme_id = theme_id
        if theme is not None:
            self.theme = theme
        self.accent1 = accent1
        self.accent2 = accent2
        self.accent3 = accent3
        self.accent4 = accent4
        self.accent5 = accent5
        self.accent6 = accent6
        self.light1 = light1
        self.light2 = light2
        self.dark1 = dark1
        self.dark2 = dark2
        self.hyperlink = hyperlink
        self.followed_hyperlink = followed_hyperlink
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def name(self):
        """Gets the name of this ThemeColorsDetails.  # noqa: E501


        :return: The name of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ThemeColorsDetails.


        :param name: The name of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def theme_id(self):
        """Gets the theme_id of this ThemeColorsDetails.  # noqa: E501


        :return: The theme_id of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._theme_id

    @theme_id.setter
    def theme_id(self, theme_id):
        """Sets the theme_id of this ThemeColorsDetails.


        :param theme_id: The theme_id of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._theme_id = theme_id

    @property
    def theme(self):
        """Gets the theme of this ThemeColorsDetails.  # noqa: E501


        :return: The theme of this ThemeColorsDetails.  # noqa: E501
        :rtype: object
        """
        return self._theme

    @theme.setter
    def theme(self, theme):
        """Sets the theme of this ThemeColorsDetails.


        :param theme: The theme of this ThemeColorsDetails.  # noqa: E501
        :type: object
        """

        self._theme = theme

    @property
    def accent1(self):
        """Gets the accent1 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent1 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent1

    @accent1.setter
    def accent1(self, accent1):
        """Sets the accent1 of this ThemeColorsDetails.


        :param accent1: The accent1 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent1 = accent1

    @property
    def accent2(self):
        """Gets the accent2 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent2 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent2

    @accent2.setter
    def accent2(self, accent2):
        """Sets the accent2 of this ThemeColorsDetails.


        :param accent2: The accent2 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent2 = accent2

    @property
    def accent3(self):
        """Gets the accent3 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent3 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent3

    @accent3.setter
    def accent3(self, accent3):
        """Sets the accent3 of this ThemeColorsDetails.


        :param accent3: The accent3 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent3 = accent3

    @property
    def accent4(self):
        """Gets the accent4 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent4 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent4

    @accent4.setter
    def accent4(self, accent4):
        """Sets the accent4 of this ThemeColorsDetails.


        :param accent4: The accent4 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent4 = accent4

    @property
    def accent5(self):
        """Gets the accent5 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent5 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent5

    @accent5.setter
    def accent5(self, accent5):
        """Sets the accent5 of this ThemeColorsDetails.


        :param accent5: The accent5 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent5 = accent5

    @property
    def accent6(self):
        """Gets the accent6 of this ThemeColorsDetails.  # noqa: E501


        :return: The accent6 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._accent6

    @accent6.setter
    def accent6(self, accent6):
        """Sets the accent6 of this ThemeColorsDetails.


        :param accent6: The accent6 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._accent6 = accent6

    @property
    def light1(self):
        """Gets the light1 of this ThemeColorsDetails.  # noqa: E501


        :return: The light1 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._light1

    @light1.setter
    def light1(self, light1):
        """Sets the light1 of this ThemeColorsDetails.


        :param light1: The light1 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._light1 = light1

    @property
    def light2(self):
        """Gets the light2 of this ThemeColorsDetails.  # noqa: E501


        :return: The light2 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._light2

    @light2.setter
    def light2(self, light2):
        """Sets the light2 of this ThemeColorsDetails.


        :param light2: The light2 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._light2 = light2

    @property
    def dark1(self):
        """Gets the dark1 of this ThemeColorsDetails.  # noqa: E501


        :return: The dark1 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._dark1

    @dark1.setter
    def dark1(self, dark1):
        """Sets the dark1 of this ThemeColorsDetails.


        :param dark1: The dark1 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._dark1 = dark1

    @property
    def dark2(self):
        """Gets the dark2 of this ThemeColorsDetails.  # noqa: E501


        :return: The dark2 of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._dark2

    @dark2.setter
    def dark2(self, dark2):
        """Sets the dark2 of this ThemeColorsDetails.


        :param dark2: The dark2 of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._dark2 = dark2

    @property
    def hyperlink(self):
        """Gets the hyperlink of this ThemeColorsDetails.  # noqa: E501


        :return: The hyperlink of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._hyperlink

    @hyperlink.setter
    def hyperlink(self, hyperlink):
        """Sets the hyperlink of this ThemeColorsDetails.


        :param hyperlink: The hyperlink of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._hyperlink = hyperlink

    @property
    def followed_hyperlink(self):
        """Gets the followed_hyperlink of this ThemeColorsDetails.  # noqa: E501


        :return: The followed_hyperlink of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._followed_hyperlink

    @followed_hyperlink.setter
    def followed_hyperlink(self, followed_hyperlink):
        """Sets the followed_hyperlink of this ThemeColorsDetails.


        :param followed_hyperlink: The followed_hyperlink of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._followed_hyperlink = followed_hyperlink

    @property
    def id(self):
        """Gets the id of this ThemeColorsDetails.  # noqa: E501


        :return: The id of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ThemeColorsDetails.


        :param id: The id of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this ThemeColorsDetails.  # noqa: E501


        :return: The date_created of this ThemeColorsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this ThemeColorsDetails.


        :param date_created: The date_created of this ThemeColorsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this ThemeColorsDetails.  # noqa: E501


        :return: The user_created of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this ThemeColorsDetails.


        :param user_created: The user_created of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this ThemeColorsDetails.  # noqa: E501


        :return: The date_modified of this ThemeColorsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this ThemeColorsDetails.


        :param date_modified: The date_modified of this ThemeColorsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this ThemeColorsDetails.  # noqa: E501


        :return: The user_modified of this ThemeColorsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this ThemeColorsDetails.


        :param user_modified: The user_modified of this ThemeColorsDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ThemeColorsDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ThemeColorsDetails):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class SharedGradientFills(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'fill_map_id': 'str',
        'angle': 'int',
        'rotate_with_shape': 'bool',
        'is_path': 'bool',
        'path_type': 'str',
        'id': 'str'
    }

    attribute_map = {
        'fill_map_id': 'fillMapId',
        'angle': 'angle',
        'rotate_with_shape': 'rotateWithShape',
        'is_path': 'isPath',
        'path_type': 'pathType',
        'id': 'id'
    }

    def __init__(self, fill_map_id=None, angle=None, rotate_with_shape=None, is_path=None, path_type=None, id=None, local_vars_configuration=None):  # noqa: E501
        """SharedGradientFills - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._fill_map_id = None
        self._angle = None
        self._rotate_with_shape = None
        self._is_path = None
        self._path_type = None
        self._id = None
        self.discriminator = None

        self.fill_map_id = fill_map_id
        self.angle = angle
        if rotate_with_shape is not None:
            self.rotate_with_shape = rotate_with_shape
        if is_path is not None:
            self.is_path = is_path
        self.path_type = path_type
        if id is not None:
            self.id = id

    @property
    def fill_map_id(self):
        """Gets the fill_map_id of this SharedGradientFills.  # noqa: E501


        :return: The fill_map_id of this SharedGradientFills.  # noqa: E501
        :rtype: str
        """
        return self._fill_map_id

    @fill_map_id.setter
    def fill_map_id(self, fill_map_id):
        """Sets the fill_map_id of this SharedGradientFills.


        :param fill_map_id: The fill_map_id of this SharedGradientFills.  # noqa: E501
        :type: str
        """

        self._fill_map_id = fill_map_id

    @property
    def angle(self):
        """Gets the angle of this SharedGradientFills.  # noqa: E501


        :return: The angle of this SharedGradientFills.  # noqa: E501
        :rtype: int
        """
        return self._angle

    @angle.setter
    def angle(self, angle):
        """Sets the angle of this SharedGradientFills.


        :param angle: The angle of this SharedGradientFills.  # noqa: E501
        :type: int
        """

        self._angle = angle

    @property
    def rotate_with_shape(self):
        """Gets the rotate_with_shape of this SharedGradientFills.  # noqa: E501


        :return: The rotate_with_shape of this SharedGradientFills.  # noqa: E501
        :rtype: bool
        """
        return self._rotate_with_shape

    @rotate_with_shape.setter
    def rotate_with_shape(self, rotate_with_shape):
        """Sets the rotate_with_shape of this SharedGradientFills.


        :param rotate_with_shape: The rotate_with_shape of this SharedGradientFills.  # noqa: E501
        :type: bool
        """

        self._rotate_with_shape = rotate_with_shape

    @property
    def is_path(self):
        """Gets the is_path of this SharedGradientFills.  # noqa: E501


        :return: The is_path of this SharedGradientFills.  # noqa: E501
        :rtype: bool
        """
        return self._is_path

    @is_path.setter
    def is_path(self, is_path):
        """Sets the is_path of this SharedGradientFills.


        :param is_path: The is_path of this SharedGradientFills.  # noqa: E501
        :type: bool
        """

        self._is_path = is_path

    @property
    def path_type(self):
        """Gets the path_type of this SharedGradientFills.  # noqa: E501


        :return: The path_type of this SharedGradientFills.  # noqa: E501
        :rtype: str
        """
        return self._path_type

    @path_type.setter
    def path_type(self, path_type):
        """Sets the path_type of this SharedGradientFills.


        :param path_type: The path_type of this SharedGradientFills.  # noqa: E501
        :type: str
        """

        self._path_type = path_type

    @property
    def id(self):
        """Gets the id of this SharedGradientFills.  # noqa: E501


        :return: The id of this SharedGradientFills.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SharedGradientFills.


        :param id: The id of this SharedGradientFills.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SharedGradientFills):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SharedGradientFills):
            return True

        return self.to_dict() != other.to_dict()



import mpld3
import typing
import presalytics.story.components
import presalytics.story.outline
import presalytics.lib.exceptions
if typing.TYPE_CHECKING:
    from matplotlib.pyplot import Figure


class MatplotlibFigure(presalytics.story.components.WidgetBase):
    __component_kind__ = 'matplotlib_figure'
    additional_properties: typing.Dict

    def __init__(self, figure: 'Figure', name: str, *args, **kwargs):
        super().__init__(self, *args, **kwargs)
        self.fig = figure
        self.name = name
        if self.fig:
            self.figure_dict = mpld3.fig_to_dict(figure)
        else:
            figure_dict = kwargs.pop("figure_dict", None)
            if figure_dict:
                self.figure_dict = figure_dict
        if self.figure_dict is None:
            message = "MatplotlibFigure requires a figure_dict attribute.  Please supply either a valid figure_dict or matplotlib.pyplot.Figure object to __init__"
            raise presalytics.lib.exceptions.ValidationError(message)
        self.figure_id = self.figure_dict["id"]
        self.additional_properties = {}
        for key, val in kwargs.items():
            self.additional_properties[key] = val
        self.outline_widget = self.serialize()

    def to_html(self):
        return '<div id="{0}" class="mpld3"></div>'.format(self.figure_id)

    @classmethod
    def deserialize(cls, outline, **kwargs):
        figure_dict = outline.data.get("figure_dict")
        return cls(None, outline.name, figure_dict=figure_dict, **outline.additional_properties)

    def serialize(self, **kwargs):
        data = {
            'figure_dict': self.figure_dict,
            'id': self.figure_id
        }
        plugin_obj = presalytics.story.outline.Plugin(
            name='mpld3',
            kind='script',
            config=data
        )
        return presalytics.story.outline.Widget(
            name=self.name,
            kind=self.__component_kind__,
            plugins=[plugin_obj.to_dict()],
            data=data,
            additional_properties=self.additional_properties
        )



# coding: utf-8

# flake8: noqa

"""
    Doc Converter

    This api converts file formats of OpenXml and OpenOffice documents formats to vector files (e.g., svg)  # noqa: E501

    The version of the OpenAPI document: 0.1
    Contact: kevin@presalytics.io
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "0.2.40"

# import apis into sdk package
from presalytics.client.presalytics_doc_converter.api.default_api import DefaultApi

# import ApiClient
from presalytics.client.presalytics_doc_converter.api_client import ApiClient
from presalytics.client.presalytics_doc_converter.configuration import Configuration
from presalytics.client.presalytics_doc_converter.exceptions import OpenApiException
from presalytics.client.presalytics_doc_converter.exceptions import ApiTypeError
from presalytics.client.presalytics_doc_converter.exceptions import ApiValueError
from presalytics.client.presalytics_doc_converter.exceptions import ApiKeyError
from presalytics.client.presalytics_doc_converter.exceptions import ApiException
# import models into sdk package
from presalytics.client.presalytics_doc_converter.models.file_to_convert import FileToConvert
from presalytics.client.presalytics_doc_converter.models.file_url import FileUrl




# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class SlideGraphicsDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'group_elements_id': 'str',
        'group_element': 'object',
        'name': 'str',
        'ooxml_id': 'int',
        'graphic_type_id': 'int',
        'height': 'int',
        'width': 'int',
        'x_offset': 'int',
        'y_offset': 'int',
        'table': 'object',
        'chart': 'object',
        'smart_art': 'object',
        'picture': 'object',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'group_elements_id': 'groupElementsId',
        'group_element': 'groupElement',
        'name': 'name',
        'ooxml_id': 'ooxmlId',
        'graphic_type_id': 'graphicTypeId',
        'height': 'height',
        'width': 'width',
        'x_offset': 'xOffset',
        'y_offset': 'yOffset',
        'table': 'table',
        'chart': 'chart',
        'smart_art': 'smartArt',
        'picture': 'picture',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, group_elements_id=None, group_element=None, name=None, ooxml_id=None, graphic_type_id=None, height=None, width=None, x_offset=None, y_offset=None, table=None, chart=None, smart_art=None, picture=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None, local_vars_configuration=None):  # noqa: E501
        """SlideGraphicsDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._group_elements_id = None
        self._group_element = None
        self._name = None
        self._ooxml_id = None
        self._graphic_type_id = None
        self._height = None
        self._width = None
        self._x_offset = None
        self._y_offset = None
        self._table = None
        self._chart = None
        self._smart_art = None
        self._picture = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.group_elements_id = group_elements_id
        if group_element is not None:
            self.group_element = group_element
        self.name = name
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        if graphic_type_id is not None:
            self.graphic_type_id = graphic_type_id
        if height is not None:
            self.height = height
        if width is not None:
            self.width = width
        if x_offset is not None:
            self.x_offset = x_offset
        if y_offset is not None:
            self.y_offset = y_offset
        if table is not None:
            self.table = table
        if chart is not None:
            self.chart = chart
        if smart_art is not None:
            self.smart_art = smart_art
        if picture is not None:
            self.picture = picture
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def group_elements_id(self):
        """Gets the group_elements_id of this SlideGraphicsDetails.  # noqa: E501

        Foreign key to the GroupElements object  # noqa: E501

        :return: The group_elements_id of this SlideGraphicsDetails.  # noqa: E501
        :rtype: str
        """
        return self._group_elements_id

    @group_elements_id.setter
    def group_elements_id(self, group_elements_id):
        """Sets the group_elements_id of this SlideGraphicsDetails.

        Foreign key to the GroupElements object  # noqa: E501

        :param group_elements_id: The group_elements_id of this SlideGraphicsDetails.  # noqa: E501
        :type: str
        """

        self._group_elements_id = group_elements_id

    @property
    def group_element(self):
        """Gets the group_element of this SlideGraphicsDetails.  # noqa: E501


        :return: The group_element of this SlideGraphicsDetails.  # noqa: E501
        :rtype: object
        """
        return self._group_element

    @group_element.setter
    def group_element(self, group_element):
        """Sets the group_element of this SlideGraphicsDetails.


        :param group_element: The group_element of this SlideGraphicsDetails.  # noqa: E501
        :type: object
        """

        self._group_element = group_element

    @property
    def name(self):
        """Gets the name of this SlideGraphicsDetails.  # noqa: E501


        :return: The name of this SlideGraphicsDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideGraphicsDetails.


        :param name: The name of this SlideGraphicsDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideGraphicsDetails.  # noqa: E501


        :return: The ooxml_id of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideGraphicsDetails.


        :param ooxml_id: The ooxml_id of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def graphic_type_id(self):
        """Gets the graphic_type_id of this SlideGraphicsDetails.  # noqa: E501


        :return: The graphic_type_id of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._graphic_type_id

    @graphic_type_id.setter
    def graphic_type_id(self, graphic_type_id):
        """Sets the graphic_type_id of this SlideGraphicsDetails.


        :param graphic_type_id: The graphic_type_id of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._graphic_type_id = graphic_type_id

    @property
    def height(self):
        """Gets the height of this SlideGraphicsDetails.  # noqa: E501


        :return: The height of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """Sets the height of this SlideGraphicsDetails.


        :param height: The height of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._height = height

    @property
    def width(self):
        """Gets the width of this SlideGraphicsDetails.  # noqa: E501


        :return: The width of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """Sets the width of this SlideGraphicsDetails.


        :param width: The width of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._width = width

    @property
    def x_offset(self):
        """Gets the x_offset of this SlideGraphicsDetails.  # noqa: E501


        :return: The x_offset of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._x_offset

    @x_offset.setter
    def x_offset(self, x_offset):
        """Sets the x_offset of this SlideGraphicsDetails.


        :param x_offset: The x_offset of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._x_offset = x_offset

    @property
    def y_offset(self):
        """Gets the y_offset of this SlideGraphicsDetails.  # noqa: E501


        :return: The y_offset of this SlideGraphicsDetails.  # noqa: E501
        :rtype: int
        """
        return self._y_offset

    @y_offset.setter
    def y_offset(self, y_offset):
        """Sets the y_offset of this SlideGraphicsDetails.


        :param y_offset: The y_offset of this SlideGraphicsDetails.  # noqa: E501
        :type: int
        """

        self._y_offset = y_offset

    @property
    def table(self):
        """Gets the table of this SlideGraphicsDetails.  # noqa: E501


        :return: The table of this SlideGraphicsDetails.  # noqa: E501
        :rtype: object
        """
        return self._table

    @table.setter
    def table(self, table):
        """Sets the table of this SlideGraphicsDetails.


        :param table: The table of this SlideGraphicsDetails.  # noqa: E501
        :type: object
        """

        self._table = table

    @property
    def chart(self):
        """Gets the chart of this SlideGraphicsDetails.  # noqa: E501


        :return: The chart of this SlideGraphicsDetails.  # noqa: E501
        :rtype: object
        """
        return self._chart

    @chart.setter
    def chart(self, chart):
        """Sets the chart of this SlideGraphicsDetails.


        :param chart: The chart of this SlideGraphicsDetails.  # noqa: E501
        :type: object
        """

        self._chart = chart

    @property
    def smart_art(self):
        """Gets the smart_art of this SlideGraphicsDetails.  # noqa: E501


        :return: The smart_art of this SlideGraphicsDetails.  # noqa: E501
        :rtype: object
        """
        return self._smart_art

    @smart_art.setter
    def smart_art(self, smart_art):
        """Sets the smart_art of this SlideGraphicsDetails.


        :param smart_art: The smart_art of this SlideGraphicsDetails.  # noqa: E501
        :type: object
        """

        self._smart_art = smart_art

    @property
    def picture(self):
        """Gets the picture of this SlideGraphicsDetails.  # noqa: E501


        :return: The picture of this SlideGraphicsDetails.  # noqa: E501
        :rtype: object
        """
        return self._picture

    @picture.setter
    def picture(self, picture):
        """Sets the picture of this SlideGraphicsDetails.


        :param picture: The picture of this SlideGraphicsDetails.  # noqa: E501
        :type: object
        """

        self._picture = picture

    @property
    def id(self):
        """Gets the id of this SlideGraphicsDetails.  # noqa: E501


        :return: The id of this SlideGraphicsDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideGraphicsDetails.


        :param id: The id of this SlideGraphicsDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SlideGraphicsDetails.  # noqa: E501


        :return: The date_created of this SlideGraphicsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SlideGraphicsDetails.


        :param date_created: The date_created of this SlideGraphicsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SlideGraphicsDetails.  # noqa: E501


        :return: The user_created of this SlideGraphicsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SlideGraphicsDetails.


        :param user_created: The user_created of this SlideGraphicsDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SlideGraphicsDetails.  # noqa: E501


        :return: The date_modified of this SlideGraphicsDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SlideGraphicsDetails.


        :param date_modified: The date_modified of this SlideGraphicsDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SlideGraphicsDetails.  # noqa: E501


        :return: The user_modified of this SlideGraphicsDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SlideGraphicsDetails.


        :param user_modified: The user_modified of this SlideGraphicsDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideGraphicsDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SlideGraphicsDetails):
            return True

        return self.to_dict() != other.to_dict()



import os
import typing
import presalytics.lib.plugins.base
import presalytics.lib.exceptions

css_path = os.path.join(os.path.dirname(__file__), "css")

local_styles_map = {
    "single_item_grid": os.path.join(css_path, "single-item-grid.css")
}


class LocalStylesPlugin(presalytics.lib.plugins.base.StylePlugin):
    __plugin_name__ = 'local'

    def to_style(self, config, **kwargs):
        id = config.get("css_file_id", None)
        if not id:
            file_path = config.get("css_file_path", None)
            if not file_path:
                message = 'Plugin requires "css_file_id" or "css_file_path" in its configuration dictionary'
                raise presalytics.lib.exceptions.MissingConfigException(message)
            if not os.path.exists(file_path):
                message = 'Path {0} in plugin configuration does not exist'.format(file_path)
                raise presalytics.lib.exceptions.MissingConfigException(message)
        else:
            file_path = local_styles_map[id]
        with open(file_path, 'r') as f:
            style_data = f.read()
        tag = "<style>{0}</style>".format(style_data)
        return tag



# coding: utf-8

# flake8: noqa
"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

# import models into model package
from presalytics.client.presalytics_ooxml_automation.models.chart_axes import ChartAxes
from presalytics.client.presalytics_ooxml_automation.models.chart_axes_details import ChartAxesDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_axis_data_types import ChartAxisDataTypes
from presalytics.client.presalytics_ooxml_automation.models.chart_chart_data import ChartChartData
from presalytics.client.presalytics_ooxml_automation.models.chart_chart_data_details import ChartChartDataDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_charts import ChartCharts
from presalytics.client.presalytics_ooxml_automation.models.chart_charts_details import ChartChartsDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_column_collections import ChartColumnCollections
from presalytics.client.presalytics_ooxml_automation.models.chart_column_collections_details import ChartColumnCollectionsDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_columns import ChartColumns
from presalytics.client.presalytics_ooxml_automation.models.chart_columns_details import ChartColumnsDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_data_points import ChartDataPoints
from presalytics.client.presalytics_ooxml_automation.models.chart_data_points_details import ChartDataPointsDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_plot_type import ChartPlotType
from presalytics.client.presalytics_ooxml_automation.models.chart_row_col import ChartRowCol
from presalytics.client.presalytics_ooxml_automation.models.chart_row_collections import ChartRowCollections
from presalytics.client.presalytics_ooxml_automation.models.chart_row_collections_details import ChartRowCollectionsDetails
from presalytics.client.presalytics_ooxml_automation.models.chart_row_name_format_types import ChartRowNameFormatTypes
from presalytics.client.presalytics_ooxml_automation.models.chart_rows import ChartRows
from presalytics.client.presalytics_ooxml_automation.models.chart_rows_details import ChartRowsDetails
from presalytics.client.presalytics_ooxml_automation.models.child_objects import ChildObjects
from presalytics.client.presalytics_ooxml_automation.models.document import Document
from presalytics.client.presalytics_ooxml_automation.models.document_details import DocumentDetails
from presalytics.client.presalytics_ooxml_automation.models.document_type import DocumentType
from presalytics.client.presalytics_ooxml_automation.models.ooxml_dto import OoxmlDTO
from presalytics.client.presalytics_ooxml_automation.models.problem_details import ProblemDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_color_transformation_attributes import SharedColorTransformationAttributes
from presalytics.client.presalytics_ooxml_automation.models.shared_color_transformation_attributes_details import SharedColorTransformationAttributesDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_color_transformations import SharedColorTransformations
from presalytics.client.presalytics_ooxml_automation.models.shared_color_transformations_details import SharedColorTransformationsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_color_types import SharedColorTypes
from presalytics.client.presalytics_ooxml_automation.models.shared_dash_types import SharedDashTypes
from presalytics.client.presalytics_ooxml_automation.models.shared_effect_attributes import SharedEffectAttributes
from presalytics.client.presalytics_ooxml_automation.models.shared_effect_attributes_details import SharedEffectAttributesDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_effect_types import SharedEffectTypes
from presalytics.client.presalytics_ooxml_automation.models.shared_effects import SharedEffects
from presalytics.client.presalytics_ooxml_automation.models.shared_effects_details import SharedEffectsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_fill_map import SharedFillMap
from presalytics.client.presalytics_ooxml_automation.models.shared_fill_map_details import SharedFillMapDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_fill_types import SharedFillTypes
from presalytics.client.presalytics_ooxml_automation.models.shared_gradient_fills import SharedGradientFills
from presalytics.client.presalytics_ooxml_automation.models.shared_gradient_fills_details import SharedGradientFillsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_gradient_stops import SharedGradientStops
from presalytics.client.presalytics_ooxml_automation.models.shared_gradient_stops_details import SharedGradientStopsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_image_fills import SharedImageFills
from presalytics.client.presalytics_ooxml_automation.models.shared_image_fills_details import SharedImageFillsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_line_end_sizes import SharedLineEndSizes
from presalytics.client.presalytics_ooxml_automation.models.shared_line_end_types import SharedLineEndTypes
from presalytics.client.presalytics_ooxml_automation.models.shared_lines import SharedLines
from presalytics.client.presalytics_ooxml_automation.models.shared_lines_details import SharedLinesDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_paragraph import SharedParagraph
from presalytics.client.presalytics_ooxml_automation.models.shared_paragraph_details import SharedParagraphDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_pictures import SharedPictures
from presalytics.client.presalytics_ooxml_automation.models.shared_pictures_details import SharedPicturesDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_solid_fills import SharedSolidFills
from presalytics.client.presalytics_ooxml_automation.models.shared_solid_fills_details import SharedSolidFillsDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_text import SharedText
from presalytics.client.presalytics_ooxml_automation.models.shared_text_container import SharedTextContainer
from presalytics.client.presalytics_ooxml_automation.models.shared_text_container_details import SharedTextContainerDetails
from presalytics.client.presalytics_ooxml_automation.models.shared_text_details import SharedTextDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_color_maps import SlideColorMaps
from presalytics.client.presalytics_ooxml_automation.models.slide_color_maps_details import SlideColorMapsDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_connector import SlideConnector
from presalytics.client.presalytics_ooxml_automation.models.slide_connector_details import SlideConnectorDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_graphic_types import SlideGraphicTypes
from presalytics.client.presalytics_ooxml_automation.models.slide_graphics import SlideGraphics
from presalytics.client.presalytics_ooxml_automation.models.slide_graphics_details import SlideGraphicsDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_group_element_types import SlideGroupElementTypes
from presalytics.client.presalytics_ooxml_automation.models.slide_group_element_types_details import SlideGroupElementTypesDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_group_elements import SlideGroupElements
from presalytics.client.presalytics_ooxml_automation.models.slide_group_elements_details import SlideGroupElementsDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_groups import SlideGroups
from presalytics.client.presalytics_ooxml_automation.models.slide_groups_details import SlideGroupsDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_shape_trees import SlideShapeTrees
from presalytics.client.presalytics_ooxml_automation.models.slide_shape_trees_details import SlideShapeTreesDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_shapes import SlideShapes
from presalytics.client.presalytics_ooxml_automation.models.slide_shapes_details import SlideShapesDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_slide_masters import SlideSlideMasters
from presalytics.client.presalytics_ooxml_automation.models.slide_slide_masters_details import SlideSlideMastersDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_slides import SlideSlides
from presalytics.client.presalytics_ooxml_automation.models.slide_slides_details import SlideSlidesDetails
from presalytics.client.presalytics_ooxml_automation.models.slide_smart_arts import SlideSmartArts
from presalytics.client.presalytics_ooxml_automation.models.slide_smart_arts_details import SlideSmartArtsDetails
from presalytics.client.presalytics_ooxml_automation.models.story_file_form_data import StoryFileFormData
from presalytics.client.presalytics_ooxml_automation.models.table_borders import TableBorders
from presalytics.client.presalytics_ooxml_automation.models.table_borders_details import TableBordersDetails
from presalytics.client.presalytics_ooxml_automation.models.table_cells import TableCells
from presalytics.client.presalytics_ooxml_automation.models.table_cells_details import TableCellsDetails
from presalytics.client.presalytics_ooxml_automation.models.table_columns import TableColumns
from presalytics.client.presalytics_ooxml_automation.models.table_columns_details import TableColumnsDetails
from presalytics.client.presalytics_ooxml_automation.models.table_rows import TableRows
from presalytics.client.presalytics_ooxml_automation.models.table_rows_details import TableRowsDetails
from presalytics.client.presalytics_ooxml_automation.models.table_tables import TableTables
from presalytics.client.presalytics_ooxml_automation.models.table_tables_details import TableTablesDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_background_fills import ThemeBackgroundFills
from presalytics.client.presalytics_ooxml_automation.models.theme_background_fills_details import ThemeBackgroundFillsDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_colors import ThemeColors
from presalytics.client.presalytics_ooxml_automation.models.theme_colors_details import ThemeColorsDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_custom_colors import ThemeCustomColors
from presalytics.client.presalytics_ooxml_automation.models.theme_custom_colors_details import ThemeCustomColorsDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_effect_map import ThemeEffectMap
from presalytics.client.presalytics_ooxml_automation.models.theme_effect_map_details import ThemeEffectMapDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_fills import ThemeFills
from presalytics.client.presalytics_ooxml_automation.models.theme_fills_details import ThemeFillsDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_fonts import ThemeFonts
from presalytics.client.presalytics_ooxml_automation.models.theme_fonts_details import ThemeFontsDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_intensity import ThemeIntensity
from presalytics.client.presalytics_ooxml_automation.models.theme_line_map import ThemeLineMap
from presalytics.client.presalytics_ooxml_automation.models.theme_line_map_details import ThemeLineMapDetails
from presalytics.client.presalytics_ooxml_automation.models.theme_themes import ThemeThemes
from presalytics.client.presalytics_ooxml_automation.models.theme_themes_details import ThemeThemesDetails



import json
import presalytics.lib.plugins.base as ext
import presalytics.lib.plugins.jinja as jinja


class Mpld3Plugin(ext.ScriptPlugin, jinja.JinjaPluginMakerMixin):
    __plugin_name__ = 'mpld3'

    __dependencies__ = [
        {
            'kind': 'script',
            'name': 'external_scripts',
            'config': {
                'approved_scripts_key': 'reveal.base'
            }
        },
        {
            'kind': 'script',
            'name': 'external_scripts',
            'config': {
                'approved_scripts_key': 'd3v3'
            }
        },
        {
            'kind': 'script',
            'name': 'external_scripts',
            'config': {
                'approved_scripts_key': 'mpld3'
            }
        }
    ]

    template = '<script type="text/javascript">mpld3.draw_figure("{{ id|safe }}",{{ fig_json|tojson }});</script>'

    def to_script(self, config, **kwargs):
        render_config = {
            'id': config["id"],
            'fig_json': config["figure_dict"]
        }
        return self.render(render_config)



"""
Runs updates to widgets in background

TODO:  This is WIP - Take a look at the daemoniker package and refactor.
"""

import subprocess
import sys
import os
import typing
import presalytics.lib.exceptions


class BackgrounderMixIn(object):
    """
    A quick and dirty implementation to "daemonize" updating of widgets.
    TODO: Some research to make this more robust and cross-platform


    A note to script kiddies: the pickled objects are never serailized
    server-side.  The only computer you'll be hacking is your own.
    """

    DETACHED_PROCESS = 8

    def __init__(self):
        self.path_to_current_interpreter = sys.executable
        if os.name == 'nt':
            self.isWindows = True
            self.process_file = "background_linux.py"
        else:
            self.isWindows = False
            self.process_file = "background_nt.pyw"

    def run_in_background(self):
        if not getattr(self, "update", None):
            message = "The {} object requires an update() method with no parameters in order to use BackgrounderMixin".format(self.__class__.__name__)
            raise presalytics.lib.exceptions.InvalidConfigurationError(message)
        else:
            update_method = getattr(self, "update")
            try:
                bound = update_method.im_self
            except AttributeError:
                message = "The update attribute on the {} object must be a bound method".format(self.__class__.__name__)
                raise presalytics.lib.exceptions.InvalidConfigurationError(message)
            if not bound:
                message = "the update method is not bound to the class and cannot be serialized"
                raise presalytics.lib.exceptions.InvalidConfigurationError(message)
        p = subprocess.run(self.process_file, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    
    @staticmethod
    def get_backgrounded_widgets() -> typing.List[str]:
        pass

    @staticmethod
    def kill_backgrounded_widget(widget_name: str) -> bool:
        pass
        
    def kill_background(self) -> bool:
        return BackgrounderMixIn.kill_backgrounded_widget(self.name)

        
            
            
        

        


import os
import shutil
import pathlib
import unittest
import typing
import presalytics
if typing.TYPE_CHECKING:
    from presalytics.client.presalytics_story import Story


class TestComponents(unittest.TestCase):
    """
    Test module features thatr render stories to dashboards or other formats
    """
    def setUp(self):
        pass

    def test_render_matplotlib_plot(self):
        test_file = os.path.join(os.path.dirname(__file__), 'files', 'matplotlib-outline.yaml')
        outline = presalytics.StoryOutline.import_yaml(test_file)
        presalytics.Revealer(outline).present(files_path='/tmp')

    def test_xml_widget(self):
        story: Story

        test_file = os.path.join(os.path.dirname(__file__), "files", "star.pptx")
        tmp_filename = os.path.join(os.path.dirname(__file__), os.path.basename(test_file))
        shutil.copyfile(test_file, tmp_filename)
        story = presalytics.create_story_from_ooxml_file(tmp_filename)
        outline = presalytics.StoryOutline.load(story.outline)
        old_widget = outline.pages[0].widgets[0]
        client = presalytics.Client()
        childs = client.ooxml_automation.documents_childobjects_get_id(old_widget.data["document_ooxml_id"])
        object_type = presalytics.OoxmlEndpointMap.shape().get_object_type()
        info = next(x for x in childs if x.object_type == object_type)
        new_color = {
            "object_name": info.entity_name,
            "hex_color": "FFFFFF"
        }
        widget = presalytics.OoxmlEditorWidget(
            name="test-editor",
            story_id=story.id,
            object_ooxml_id=info.entity_id,
            endpoint_id=info.object_type.split(".")[1],
            transform_class=presalytics.ChangeShapeColor,
            transform_params=new_color
        )
        outline.pages[0].widgets[0] = widget.outline_widget
        presalytics.COMPONENTS.register(widget)
        presalytics.Revealer(outline).present()
        os.remove(tmp_filename)

    def test_file_widget(self):
        test_file = os.path.join(os.path.dirname(__file__), "files", "star.pptx")
        tmp_filename = os.path.join(os.path.dirname(__file__), os.path.basename(test_file))
        shutil.copyfile(test_file, tmp_filename)
        story = presalytics.create_story_from_ooxml_file(tmp_filename)
        pathlib.Path(tmp_filename).touch()
        story_outline = presalytics.StoryOutline.load(story.outline)
        widget_data = story_outline.pages[0].widgets[0]
        presalytics.OoxmlFileWidget.deserialize(widget_data)
        presalytics.Revealer(story_outline).present()

    def tearDown(self):
        pass



"""
Presalytics python client
https://presalytics.io
"""
import os
import environs
import logging
import presalytics.lib
import presalytics.lib.logger
import presalytics.lib.plugins
import presalytics.lib.plugins.base
import presalytics.lib.config_loader
import presalytics.story
import presalytics.story.components
from presalytics.client.api import Client
from presalytics.lib.plugins.base import PluginBase
from presalytics.lib.plugins.external import ApprovedExternalLinks, ApprovedExternalScripts
from presalytics.lib.plugins.jinja import JinjaPluginMakerMixin
from presalytics.lib.plugins.local import LocalStylesPlugin
from presalytics.lib.plugins.matplotlib import Mpld3Plugin
from presalytics.lib.plugins.ooxml import OoxmlTheme
from presalytics.lib.plugins.reveal import RevealConfigPlugin
from presalytics.lib.plugins.reveal_theme import RevealCustomTheme
from presalytics.lib.templates.base import JinjaTemplateBuilder
from presalytics.lib.widgets.matplotlib import MatplotlibFigure
from presalytics.lib.widgets.ooxml import OoxmlFileWidget, OoxmlEndpointMap
from presalytics.story.outline import StoryOutline
from presalytics.story.revealer import Revealer
from presalytics.story.components import WidgetBase, PageTemplateBase, Renderer, ThemeBase
from presalytics.lib.tools.ooxml_tools import (
    create_theme_from_ooxml_document,
    create_pages_from_ooxml_document,
    create_story_from_ooxml_file,
    create_outline_from_ooxml_document
)
from presalytics.lib.widgets.ooxml_editors import (
    OoxmlEditorWidget,
    XmlTransformBase,
    ChangeShapeColor
)

__all__ = [
    'CONFIG',
    'COMPONENTS',
    'PLUGINS',
    'Client',
    'PluginBase',
    'ApprovedExternalLinks',
    'ApprovedExternalScripts',
    'JinjaPluginMakerMixin',
    'LocalStylesPlugin',
    'Mpld3Plugin',
    'OoxmlTheme',
    'RevealConfigPlugin',
    'RevealCustomTheme',
    'JinjaTemplateBuilder',
    'MatplotlibFigure',
    'OoxmlFileWidget',
    'OoxmlEndpointMap',
    'StoryOutline',
    'Revealer',
    'WidgetBase',
    'PageTemplateBase',
    'Renderer',
    'ThemeBase',
    'create_theme_from_ooxml_document',
    'create_pages_from_document',
    'create_outline_from_ooxml_file',
    'OoxmlEditorWidget',
    'XmlTransformBase',
    'ChangeShapeColor'
]

env = environs.Env()
env.read_env()


# A comma-separated list of paths to search of config.py, plugin classes, and component classes
autodiscover_paths = env.list('AUTODISCOVER_PATHS', [])

CONFIG = presalytics.lib.config_loader.load_config(additional_paths=autodiscover_paths)

file_logger = CONFIG.get("USE_LOGGER", True)
log_level = CONFIG.get("LOG_LEVEL", logging.DEBUG)

presalytics.lib.logger.configure_logger(log_level=log_level, file_logger=file_logger)

registry_kwargs = {
    'show_errors': False,
    'autodiscover_paths': autodiscover_paths
}

PLUGINS = presalytics.lib.plugins.base.PluginRegistry(**registry_kwargs)

COMPONENTS = presalytics.story.components.ComponentRegistry(**registry_kwargs)



import sass
import os
import presalytics.lib.plugins.reveal_theme


class OoxmlTheme(presalytics.lib.plugins.reveal_theme.RevealCustomTheme):
    __plugin_name__ = 'ooxml-theme'

    def to_style(self, config, **kwargs):
        base_path = os.path.join(os.path.dirname(__file__), 'scss')
        scss_file = os.path.join(base_path, 'overrides.tmpl')
        config.update({'path': base_path})
        with open(scss_file, 'r') as file:
            scss_template_string = file.read()
        scss_string = scss_template_string.format(**config)
        new_css = sass.compile(string=scss_string)
        style_string = "<style>\n{0}\n</style>".format(new_css)
        font_names = [config["headingFont"], config["bodyFont"]]
        links = self.get_fonts(font_names)
        return links + style_string



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class TableBordersDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cell_id': 'str',
        'cell': 'object',
        'top': 'object',
        'bottom': 'object',
        'right': 'object',
        'left': 'object',
        't_lto_br': 'object',
        'b_lto_tr': 'object',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'cell_id': 'cellId',
        'cell': 'cell',
        'top': 'top',
        'bottom': 'bottom',
        'right': 'right',
        'left': 'left',
        't_lto_br': 'tLtoBR',
        'b_lto_tr': 'bLtoTR',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, cell_id=None, cell=None, top=None, bottom=None, right=None, left=None, t_lto_br=None, b_lto_tr=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None, local_vars_configuration=None):  # noqa: E501
        """TableBordersDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cell_id = None
        self._cell = None
        self._top = None
        self._bottom = None
        self._right = None
        self._left = None
        self._t_lto_br = None
        self._b_lto_tr = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.cell_id = cell_id
        if cell is not None:
            self.cell = cell
        if top is not None:
            self.top = top
        if bottom is not None:
            self.bottom = bottom
        if right is not None:
            self.right = right
        if left is not None:
            self.left = left
        if t_lto_br is not None:
            self.t_lto_br = t_lto_br
        if b_lto_tr is not None:
            self.b_lto_tr = b_lto_tr
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def cell_id(self):
        """Gets the cell_id of this TableBordersDetails.  # noqa: E501


        :return: The cell_id of this TableBordersDetails.  # noqa: E501
        :rtype: str
        """
        return self._cell_id

    @cell_id.setter
    def cell_id(self, cell_id):
        """Sets the cell_id of this TableBordersDetails.


        :param cell_id: The cell_id of this TableBordersDetails.  # noqa: E501
        :type: str
        """

        self._cell_id = cell_id

    @property
    def cell(self):
        """Gets the cell of this TableBordersDetails.  # noqa: E501


        :return: The cell of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._cell

    @cell.setter
    def cell(self, cell):
        """Sets the cell of this TableBordersDetails.


        :param cell: The cell of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._cell = cell

    @property
    def top(self):
        """Gets the top of this TableBordersDetails.  # noqa: E501


        :return: The top of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._top

    @top.setter
    def top(self, top):
        """Sets the top of this TableBordersDetails.


        :param top: The top of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._top = top

    @property
    def bottom(self):
        """Gets the bottom of this TableBordersDetails.  # noqa: E501


        :return: The bottom of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._bottom

    @bottom.setter
    def bottom(self, bottom):
        """Sets the bottom of this TableBordersDetails.


        :param bottom: The bottom of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._bottom = bottom

    @property
    def right(self):
        """Gets the right of this TableBordersDetails.  # noqa: E501


        :return: The right of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._right

    @right.setter
    def right(self, right):
        """Sets the right of this TableBordersDetails.


        :param right: The right of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._right = right

    @property
    def left(self):
        """Gets the left of this TableBordersDetails.  # noqa: E501


        :return: The left of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._left

    @left.setter
    def left(self, left):
        """Sets the left of this TableBordersDetails.


        :param left: The left of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._left = left

    @property
    def t_lto_br(self):
        """Gets the t_lto_br of this TableBordersDetails.  # noqa: E501


        :return: The t_lto_br of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._t_lto_br

    @t_lto_br.setter
    def t_lto_br(self, t_lto_br):
        """Sets the t_lto_br of this TableBordersDetails.


        :param t_lto_br: The t_lto_br of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._t_lto_br = t_lto_br

    @property
    def b_lto_tr(self):
        """Gets the b_lto_tr of this TableBordersDetails.  # noqa: E501


        :return: The b_lto_tr of this TableBordersDetails.  # noqa: E501
        :rtype: object
        """
        return self._b_lto_tr

    @b_lto_tr.setter
    def b_lto_tr(self, b_lto_tr):
        """Sets the b_lto_tr of this TableBordersDetails.


        :param b_lto_tr: The b_lto_tr of this TableBordersDetails.  # noqa: E501
        :type: object
        """

        self._b_lto_tr = b_lto_tr

    @property
    def id(self):
        """Gets the id of this TableBordersDetails.  # noqa: E501


        :return: The id of this TableBordersDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this TableBordersDetails.


        :param id: The id of this TableBordersDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this TableBordersDetails.  # noqa: E501


        :return: The date_created of this TableBordersDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this TableBordersDetails.


        :param date_created: The date_created of this TableBordersDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this TableBordersDetails.  # noqa: E501


        :return: The user_created of this TableBordersDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this TableBordersDetails.


        :param user_created: The user_created of this TableBordersDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this TableBordersDetails.  # noqa: E501


        :return: The date_modified of this TableBordersDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this TableBordersDetails.


        :param date_modified: The date_modified of this TableBordersDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this TableBordersDetails.  # noqa: E501


        :return: The user_modified of this TableBordersDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this TableBordersDetails.


        :param user_modified: The user_modified of this TableBordersDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TableBordersDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TableBordersDetails):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

# flake8: noqa

"""
    Story

    This API is the main entry point for creating, editing and publishing analytics throught the Presalytics API  # noqa: E501

    The version of the OpenAPI document: 0.3.1
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

__version__ = "0.1.46"

# import apis into sdk package
from presalytics.client.presalytics_story.api.default_api import DefaultApi

# import ApiClient
from presalytics.client.presalytics_story.api_client import ApiClient
from presalytics.client.presalytics_story.configuration import Configuration
from presalytics.client.presalytics_story.exceptions import OpenApiException
from presalytics.client.presalytics_story.exceptions import ApiTypeError
from presalytics.client.presalytics_story.exceptions import ApiValueError
from presalytics.client.presalytics_story.exceptions import ApiKeyError
from presalytics.client.presalytics_story.exceptions import ApiException
# import models into sdk package
from presalytics.client.presalytics_story.models.base_model import BaseModel
from presalytics.client.presalytics_story.models.ooxml_document import OoxmlDocument
from presalytics.client.presalytics_story.models.ooxml_document_all_of import OoxmlDocumentAllOf
from presalytics.client.presalytics_story.models.outline import Outline
from presalytics.client.presalytics_story.models.permission_type import PermissionType
from presalytics.client.presalytics_story.models.permission_type_all_of import PermissionTypeAllOf
from presalytics.client.presalytics_story.models.problem_detail import ProblemDetail
from presalytics.client.presalytics_story.models.story import Story
from presalytics.client.presalytics_story.models.story_all_of import StoryAllOf
from presalytics.client.presalytics_story.models.story_collaborator import StoryCollaborator
from presalytics.client.presalytics_story.models.story_collaborator_all_of import StoryCollaboratorAllOf
from presalytics.client.presalytics_story.models.story_outline_history import StoryOutlineHistory
from presalytics.client.presalytics_story.models.story_outline_history_all_of import StoryOutlineHistoryAllOf




# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class TableCells(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'row_id': 'str',
        'column_id': 'str',
        'row_span': 'int',
        'column_span': 'int',
        'is_merged_vertical': 'bool',
        'is_merged_horozontal': 'bool',
        'id': 'str'
    }

    attribute_map = {
        'row_id': 'rowId',
        'column_id': 'columnId',
        'row_span': 'rowSpan',
        'column_span': 'columnSpan',
        'is_merged_vertical': 'isMergedVertical',
        'is_merged_horozontal': 'isMergedHorozontal',
        'id': 'id'
    }

    def __init__(self, row_id=None, column_id=None, row_span=None, column_span=None, is_merged_vertical=None, is_merged_horozontal=None, id=None, local_vars_configuration=None):  # noqa: E501
        """TableCells - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._row_id = None
        self._column_id = None
        self._row_span = None
        self._column_span = None
        self._is_merged_vertical = None
        self._is_merged_horozontal = None
        self._id = None
        self.discriminator = None

        self.row_id = row_id
        self.column_id = column_id
        if row_span is not None:
            self.row_span = row_span
        if column_span is not None:
            self.column_span = column_span
        if is_merged_vertical is not None:
            self.is_merged_vertical = is_merged_vertical
        if is_merged_horozontal is not None:
            self.is_merged_horozontal = is_merged_horozontal
        if id is not None:
            self.id = id

    @property
    def row_id(self):
        """Gets the row_id of this TableCells.  # noqa: E501


        :return: The row_id of this TableCells.  # noqa: E501
        :rtype: str
        """
        return self._row_id

    @row_id.setter
    def row_id(self, row_id):
        """Sets the row_id of this TableCells.


        :param row_id: The row_id of this TableCells.  # noqa: E501
        :type: str
        """

        self._row_id = row_id

    @property
    def column_id(self):
        """Gets the column_id of this TableCells.  # noqa: E501


        :return: The column_id of this TableCells.  # noqa: E501
        :rtype: str
        """
        return self._column_id

    @column_id.setter
    def column_id(self, column_id):
        """Sets the column_id of this TableCells.


        :param column_id: The column_id of this TableCells.  # noqa: E501
        :type: str
        """

        self._column_id = column_id

    @property
    def row_span(self):
        """Gets the row_span of this TableCells.  # noqa: E501


        :return: The row_span of this TableCells.  # noqa: E501
        :rtype: int
        """
        return self._row_span

    @row_span.setter
    def row_span(self, row_span):
        """Sets the row_span of this TableCells.


        :param row_span: The row_span of this TableCells.  # noqa: E501
        :type: int
        """

        self._row_span = row_span

    @property
    def column_span(self):
        """Gets the column_span of this TableCells.  # noqa: E501


        :return: The column_span of this TableCells.  # noqa: E501
        :rtype: int
        """
        return self._column_span

    @column_span.setter
    def column_span(self, column_span):
        """Sets the column_span of this TableCells.


        :param column_span: The column_span of this TableCells.  # noqa: E501
        :type: int
        """

        self._column_span = column_span

    @property
    def is_merged_vertical(self):
        """Gets the is_merged_vertical of this TableCells.  # noqa: E501


        :return: The is_merged_vertical of this TableCells.  # noqa: E501
        :rtype: bool
        """
        return self._is_merged_vertical

    @is_merged_vertical.setter
    def is_merged_vertical(self, is_merged_vertical):
        """Sets the is_merged_vertical of this TableCells.


        :param is_merged_vertical: The is_merged_vertical of this TableCells.  # noqa: E501
        :type: bool
        """

        self._is_merged_vertical = is_merged_vertical

    @property
    def is_merged_horozontal(self):
        """Gets the is_merged_horozontal of this TableCells.  # noqa: E501


        :return: The is_merged_horozontal of this TableCells.  # noqa: E501
        :rtype: bool
        """
        return self._is_merged_horozontal

    @is_merged_horozontal.setter
    def is_merged_horozontal(self, is_merged_horozontal):
        """Sets the is_merged_horozontal of this TableCells.


        :param is_merged_horozontal: The is_merged_horozontal of this TableCells.  # noqa: E501
        :type: bool
        """

        self._is_merged_horozontal = is_merged_horozontal

    @property
    def id(self):
        """Gets the id of this TableCells.  # noqa: E501


        :return: The id of this TableCells.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this TableCells.


        :param id: The id of this TableCells.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TableCells):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TableCells):
            return True

        return self.to_dict() != other.to_dict()



import abc
import typing
import os
import logging
import presalytics.lib
import presalytics.lib.registry
import presalytics.lib.exceptions
if typing.TYPE_CHECKING:
    from presalytics.story.outline import Widget, Page, Plugin, OutlineBase


logger = logging.getLogger("presalytics.story.components")


class ComponentBase(abc.ABC):
    """
    This class serves as the primary interface clas for the story components.  Instances of
    this interface carry enough information to rendered to html.  Developers, analysts and
    users should inherit from base class if they want components to be registered in the
    rendering pipeline for presalytics stories.

    Instance of this class must:
    1. Initialize with a name parameter
    2  Serialize to a subclass of presalytics.story.outline.OutlineBase
    3. Deserialize from an instance of presalytics.story.OutlineBase
    4. Render to html

    The __component_type__ and __component_name__ metadata on subclasses is required in
    order for instances to be registered in rendering pipeline for automatic updates

    Attributes:
    ----------

    name: str
        The name of the instance of the component.  Used in component as a key to lookup
        instances to re-calcute during rendering

    css: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_STYLES dictionary
        in the presalytics.lib.templates.base module.

    js: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_SCRIPTS dictionary
        in the presalytics.lib.templates.base module.

    __component_type__: str
        The idenifier for the component superclass (e.g. widget, page_template, renderer, theme).
        Used for component registration.

    __component_kind__:
        An identifier for this component class.  Used for component registration.
    """
    __component_type__: str
    __component_kind__: str
    __plugins__: typing.List[str]
    name: str

    __plugins__ = []

    def __init__(self, name, *args, **kwargs):
        self.name = name

    @abc.abstractmethod
    def render(self, **kwargs):
        raise NotImplementedError

    @abc.abstractmethod
    def serialize(self):
        raise NotImplementedError

    @classmethod
    def deserialize(cls, component: typing.Type['OutlineBase'], **kwargs):
        raise NotImplementedError


class WidgetBase(ComponentBase):
    """
    Inherit from this base class to create widget components that can be rendered to html via the
    presalytics.story.revealer.Revealer class.  This component also need to build a method
    that allows the widget to be serialzed into a presalytics.story.outline.Widget object.

    Parameters:
    ----------
    widget: Widget
        A presalytics.story.outline.Widget object use for initialized the component class.

    Attributes:
    ----------
    outline_widget: Widget
        A presalytics.story.outline.Widget object

    css: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_STYLES dictionary
        in the presalytics.lib.templates.base module.

    js: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_SCRIPTS dictionary
        in the presalytics.lib.templates.base module.
    """
    outline_widget: typing.Optional['Widget']

    __component_type__ = 'widget'

    def __init__(self, name, *args, **kwargs) -> None:
        """
        Note: When inheriting from this base class, call "super().__init__(self, *args, **kwargs)" before add your
        own custom initialization.

        """
        super().__init__(self, name, *args, **kwargs)
        self.outline_widget = None

    def render(self, component, **kwargs):
        self.to_html(component, None, **kwargs)

    @abc.abstractmethod
    def to_html(self, data: typing.Dict = None, **kwargs) -> str:
        """
        Returns valid html that renders the widget in a broswer.

        Parameters:
        ----------
        data: Dict
            The data parameter is a dictionary should contain the minimum amount of that is required to
            successfully render the object.  As the widget is update, data control how the disply of
            information changes.
        **kwargs:
            Optional keyword arguments can be used in subclass to modify the behavior of the to_html function.
            these keyword arguments should be geinvariant through successive updates to the chart.  For example,
            keycloak argument should control the styling of the widget, which should not change as the data in
            the object (e.g., a chart) is updated.  Keyword arguments are loaded via additional_properties
            parameter in in the presalytics.story.outline.Widget object.

        Returns
        ----------
        A string of containing an html fragment that will be loaded into a template in successive operations
        """
        raise NotImplementedError

    @abc.abstractmethod
    def serialize(self, **kwargs) -> 'Widget':
        """
        Creates presalytics.story.outline.Widget object from instance data. This widget should
        have the correct name, data and additional_properties so the same widget can be reconstituted
        via the to_html method, given the same set of data.

        Typically, this method will call an update method that run a local script with updates this
        Widget's data Dictionary prior being loading into the Widget outline object for serialization.

        Parameters:
        ----------
        **kwargs:
            Optional keyword arguments can be used in subclass to modify the behavior of the to_html function.
            these keyword arguments should be be invariant through successive updates to the chart. Overrides
            for this widgets default additional_properties shoudl be loaded via these keyword argments.

        """
        raise NotImplementedError

    @classmethod
    def deserialize(cls, widget: 'Widget', **kwargs) -> 'WidgetBase':
        """
        Creates an instance of the widget from the data object in the presalytics.story.outline.Widget
        object. This method exists to ensure widgets can be portable across environments.  To clarify,
        widgets built on the client-side via the __init__ method can be reconstructed server-side via
        the deserialize method.  This allows decoupling of the widget generation/updating of data and
        the rendering of the widget in a UI.  Renderers (e.g., presalytics.story.revealer.Revealer object)
        need not know about how the data get updated, but can update the graphic with data generated by
        the widget when the serialize method is called.

        Parameters:
        ----------

        widget: Widget
            A prealytics.story.outline.Widget object

        Returns:
        ----------

        An instance the widget class
        """
        raise NotImplementedError


class PageTemplateBase(ComponentBase):
    """
    Inherit from this base class to render templates to html via the
    presalytics.story.revealer.Revealer class.

    Parameters:
    ----------
    page: Page
        A presalytics.story.outline.Page object for instalizing the class

    Attributes:
    ----------
    outline_page: Page
        A presalytics.story.outline.Page object

    widgets: List[WidgetComponentBase]
        A list widget that will be loaded into templates and rendered via placeholders.
        These widgets must have a "to_html(self, data, **kwargs)" method.

    css: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_STYLES dictionary
        in the presalytics.lib.templates.base module.

    js: List[str]
        A list of keys in dot notation that point to links in the ALLOWED_SCRIPTS dictionary
        in the presalytics.lib.templates.base module.
    """

    outline_page: 'Page'
    widgets: typing.List['WidgetBase']
    plugins: typing.List[typing.Dict]

    __component_type__ = 'page'

    def __init__(self, page: 'Page') -> None:
        self.outline_page = page
        self.widgets = self.get_page_widgets(self.outline_page)

    @abc.abstractmethod
    def render(self, **kwargs) -> str:
        """
        Returns valid html that renders the template in a broswer with data loaded from widgets.

        Parameters:
        ----------
        widgets: Sequence[WidgetComponentBase]
            List of widget instances a one to many different class that inhereit from the WidgetComponentBase
            abstract class. Defaults tot he widget list that the class as initilazed with.

        **kwargs:
            Optional keyword arguments can be used in subclass to modify the behavior of the to_html function.
            these keyword arguments should be geinvariant through successive updates to the chart.  For example,
            keycloak argument should control the styling of the widget, which should not change as the data in
            the object (e.g., a chart) is updated.  Keyword arguments are loaded via additional_properties
            parameter in in the presalytics.story.outline.Widget object.

        Returns
        ----------
        A string of containing an html fragment that will be loaded into a template in successive operations
        """
        raise NotImplementedError

    def load_widget(self, widget: 'Widget'):
        """
        Converts a presalytics.story.outline.Widget object to a subclass of WidgetComponentBase
        via a presalytics.story.loaders.WidgetLoaderBase object.
        """
        class_key = "widget." + widget.kind
        key = class_key + "." + widget.name
        if presalytics.COMPONENTS.get_instance(key):
            widget_instance = presalytics.COMPONENTS.get_instance(key)
        else:
            klass = presalytics.COMPONENTS.get(class_key)
            deserialize_method = getattr(klass, "deserialize", None)
            if callable(deserialize_method):
                widget_instance = deserialize_method(widget)
            else:
                message = "Widget component instance or class (kind) {0} unavailable in component registry".format(key)
                raise presalytics.lib.exceptions.MissingConfigException(message)
        return widget_instance

    def get_page_widgets(self, page: 'Page'):
        """
        Converts the widgets within a presaltytics.story.outline.Page object to a list
        of widgets subclassed from WidgetComponentBase
        """
        widget_instances = []
        for widget_outline in page.widgets:
            next_widget = self.load_widget(widget_outline)
            widget_instances.append(next_widget)
        return widget_instances
    
    def serialize(self):
        return self.outline_page.dump()

    @classmethod
    def deseriailize(cls, component, **kwargs):
        return cls(page=component)


class ThemeBase(ComponentBase):
    """
    Themes are containers for plugins should be rendered once
    across the entire document.  The init method should configure
    parameters with get passed to plugins via serialization and 
    deserialization.
    """
    plugins: typing.Optional[typing.List['Plugin']]

    __component_type__ = 'theme'

    def __init__(self, **kwargs):
        pass

    def render(self, **kwargs):
        pass


class Renderer(ComponentBase):
    __component_type__ = 'renderer'


class ComponentRegistry(presalytics.lib.registry.RegistryBase):
    def __init__(self, **kwargs):
        self.instances = {}
        super(ComponentRegistry, self).__init__(**kwargs)

    def get_type(self, klass):
        return getattr(klass, "__component_type__", None)

    def get_name(self, klass):
        return getattr(klass, "__component_kind__", None)

    def get_instance_name(self, klass):
        return getattr(klass, "name", None)

    def load_class(self, klass):
        super().load_class(klass)
        if isinstance(klass, ComponentBase):
            klass_type = self.get_type(klass)
            try:
                if klass_type:
                    klass_name = self.get_name(klass)
                    instance_name = self.get_instance_name(klass)
                    if instance_name and klass_name:
                        key = "{0}.{1}.{2}".format(klass_type, klass_name, instance_name)
                        if key not in self.instances.keys():
                            self.instances[key] = klass
                    else:
                        message = '{0} instance missing "__component_kind__" or "name" attribute'.format(klass_type)
                        logger.error()
            except Exception:
                message = "Unable to register instance {0} with type {1}".format(klass.__name__, klass_type)
                logger.error(message)

    def get_instance(self, key):
        return self.instances.get(key, None)



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class SlideGraphics(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'group_elements_id': 'str',
        'name': 'str',
        'ooxml_id': 'int',
        'graphic_type_id': 'int',
        'height': 'int',
        'width': 'int',
        'x_offset': 'int',
        'y_offset': 'int',
        'id': 'str'
    }

    attribute_map = {
        'group_elements_id': 'groupElementsId',
        'name': 'name',
        'ooxml_id': 'ooxmlId',
        'graphic_type_id': 'graphicTypeId',
        'height': 'height',
        'width': 'width',
        'x_offset': 'xOffset',
        'y_offset': 'yOffset',
        'id': 'id'
    }

    def __init__(self, group_elements_id=None, name=None, ooxml_id=None, graphic_type_id=None, height=None, width=None, x_offset=None, y_offset=None, id=None, local_vars_configuration=None):  # noqa: E501
        """SlideGraphics - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._group_elements_id = None
        self._name = None
        self._ooxml_id = None
        self._graphic_type_id = None
        self._height = None
        self._width = None
        self._x_offset = None
        self._y_offset = None
        self._id = None
        self.discriminator = None

        self.group_elements_id = group_elements_id
        self.name = name
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        if graphic_type_id is not None:
            self.graphic_type_id = graphic_type_id
        if height is not None:
            self.height = height
        if width is not None:
            self.width = width
        if x_offset is not None:
            self.x_offset = x_offset
        if y_offset is not None:
            self.y_offset = y_offset
        if id is not None:
            self.id = id

    @property
    def group_elements_id(self):
        """Gets the group_elements_id of this SlideGraphics.  # noqa: E501


        :return: The group_elements_id of this SlideGraphics.  # noqa: E501
        :rtype: str
        """
        return self._group_elements_id

    @group_elements_id.setter
    def group_elements_id(self, group_elements_id):
        """Sets the group_elements_id of this SlideGraphics.


        :param group_elements_id: The group_elements_id of this SlideGraphics.  # noqa: E501
        :type: str
        """

        self._group_elements_id = group_elements_id

    @property
    def name(self):
        """Gets the name of this SlideGraphics.  # noqa: E501


        :return: The name of this SlideGraphics.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideGraphics.


        :param name: The name of this SlideGraphics.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideGraphics.  # noqa: E501


        :return: The ooxml_id of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideGraphics.


        :param ooxml_id: The ooxml_id of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def graphic_type_id(self):
        """Gets the graphic_type_id of this SlideGraphics.  # noqa: E501


        :return: The graphic_type_id of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._graphic_type_id

    @graphic_type_id.setter
    def graphic_type_id(self, graphic_type_id):
        """Sets the graphic_type_id of this SlideGraphics.


        :param graphic_type_id: The graphic_type_id of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._graphic_type_id = graphic_type_id

    @property
    def height(self):
        """Gets the height of this SlideGraphics.  # noqa: E501


        :return: The height of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._height

    @height.setter
    def height(self, height):
        """Sets the height of this SlideGraphics.


        :param height: The height of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._height = height

    @property
    def width(self):
        """Gets the width of this SlideGraphics.  # noqa: E501


        :return: The width of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._width

    @width.setter
    def width(self, width):
        """Sets the width of this SlideGraphics.


        :param width: The width of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._width = width

    @property
    def x_offset(self):
        """Gets the x_offset of this SlideGraphics.  # noqa: E501


        :return: The x_offset of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._x_offset

    @x_offset.setter
    def x_offset(self, x_offset):
        """Sets the x_offset of this SlideGraphics.


        :param x_offset: The x_offset of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._x_offset = x_offset

    @property
    def y_offset(self):
        """Gets the y_offset of this SlideGraphics.  # noqa: E501


        :return: The y_offset of this SlideGraphics.  # noqa: E501
        :rtype: int
        """
        return self._y_offset

    @y_offset.setter
    def y_offset(self, y_offset):
        """Sets the y_offset of this SlideGraphics.


        :param y_offset: The y_offset of this SlideGraphics.  # noqa: E501
        :type: int
        """

        self._y_offset = y_offset

    @property
    def id(self):
        """Gets the id of this SlideGraphics.  # noqa: E501


        :return: The id of this SlideGraphics.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideGraphics.


        :param id: The id of this SlideGraphics.  # noqa: E501
        :type: str
        """

        self._id = id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideGraphics):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SlideGraphics):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    OOXML Automation

    This API helps users convert Excel and Powerpoint documents into rich, live dashboards and stories.  # noqa: E501

    The version of the OpenAPI document: 0.1.0-no-tags
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from presalytics.client.presalytics_ooxml_automation.configuration import Configuration


class SlideSlidesDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'document_id': 'str',
        'document': 'object',
        'slide_master': 'object',
        'number': 'int',
        'ooxml_id': 'int',
        'svg_blob_url': 'str',
        'slide_document_url': 'str',
        'theme': 'object',
        'shape_tree': 'object',
        'base_element_blob_url': 'str',
        'changed_base_element_blob_url': 'str',
        'package_uri': 'str',
        'name': 'str',
        'id': 'str',
        'date_created': 'datetime',
        'user_created': 'str',
        'date_modified': 'datetime',
        'user_modified': 'str'
    }

    attribute_map = {
        'document_id': 'documentId',
        'document': 'document',
        'slide_master': 'slideMaster',
        'number': 'number',
        'ooxml_id': 'ooxmlId',
        'svg_blob_url': 'svgBlobUrl',
        'slide_document_url': 'slideDocumentUrl',
        'theme': 'theme',
        'shape_tree': 'shapeTree',
        'base_element_blob_url': 'baseElementBlobUrl',
        'changed_base_element_blob_url': 'changedBaseElementBlobUrl',
        'package_uri': 'packageUri',
        'name': 'name',
        'id': 'id',
        'date_created': 'dateCreated',
        'user_created': 'userCreated',
        'date_modified': 'dateModified',
        'user_modified': 'userModified'
    }

    def __init__(self, document_id=None, document=None, slide_master=None, number=None, ooxml_id=None, svg_blob_url=None, slide_document_url=None, theme=None, shape_tree=None, base_element_blob_url=None, changed_base_element_blob_url=None, package_uri=None, name=None, id=None, date_created=None, user_created=None, date_modified=None, user_modified=None, local_vars_configuration=None):  # noqa: E501
        """SlideSlidesDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._document_id = None
        self._document = None
        self._slide_master = None
        self._number = None
        self._ooxml_id = None
        self._svg_blob_url = None
        self._slide_document_url = None
        self._theme = None
        self._shape_tree = None
        self._base_element_blob_url = None
        self._changed_base_element_blob_url = None
        self._package_uri = None
        self._name = None
        self._id = None
        self._date_created = None
        self._user_created = None
        self._date_modified = None
        self._user_modified = None
        self.discriminator = None

        self.document_id = document_id
        if document is not None:
            self.document = document
        if slide_master is not None:
            self.slide_master = slide_master
        if number is not None:
            self.number = number
        if ooxml_id is not None:
            self.ooxml_id = ooxml_id
        self.svg_blob_url = svg_blob_url
        self.slide_document_url = slide_document_url
        if theme is not None:
            self.theme = theme
        if shape_tree is not None:
            self.shape_tree = shape_tree
        self.base_element_blob_url = base_element_blob_url
        self.changed_base_element_blob_url = changed_base_element_blob_url
        self.package_uri = package_uri
        self.name = name
        if id is not None:
            self.id = id
        if date_created is not None:
            self.date_created = date_created
        if user_created is not None:
            self.user_created = user_created
        if date_modified is not None:
            self.date_modified = date_modified
        if user_modified is not None:
            self.user_modified = user_modified

    @property
    def document_id(self):
        """Gets the document_id of this SlideSlidesDetails.  # noqa: E501


        :return: The document_id of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """Sets the document_id of this SlideSlidesDetails.


        :param document_id: The document_id of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._document_id = document_id

    @property
    def document(self):
        """Gets the document of this SlideSlidesDetails.  # noqa: E501


        :return: The document of this SlideSlidesDetails.  # noqa: E501
        :rtype: object
        """
        return self._document

    @document.setter
    def document(self, document):
        """Sets the document of this SlideSlidesDetails.


        :param document: The document of this SlideSlidesDetails.  # noqa: E501
        :type: object
        """

        self._document = document

    @property
    def slide_master(self):
        """Gets the slide_master of this SlideSlidesDetails.  # noqa: E501


        :return: The slide_master of this SlideSlidesDetails.  # noqa: E501
        :rtype: object
        """
        return self._slide_master

    @slide_master.setter
    def slide_master(self, slide_master):
        """Sets the slide_master of this SlideSlidesDetails.


        :param slide_master: The slide_master of this SlideSlidesDetails.  # noqa: E501
        :type: object
        """

        self._slide_master = slide_master

    @property
    def number(self):
        """Gets the number of this SlideSlidesDetails.  # noqa: E501


        :return: The number of this SlideSlidesDetails.  # noqa: E501
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """Sets the number of this SlideSlidesDetails.


        :param number: The number of this SlideSlidesDetails.  # noqa: E501
        :type: int
        """

        self._number = number

    @property
    def ooxml_id(self):
        """Gets the ooxml_id of this SlideSlidesDetails.  # noqa: E501


        :return: The ooxml_id of this SlideSlidesDetails.  # noqa: E501
        :rtype: int
        """
        return self._ooxml_id

    @ooxml_id.setter
    def ooxml_id(self, ooxml_id):
        """Sets the ooxml_id of this SlideSlidesDetails.


        :param ooxml_id: The ooxml_id of this SlideSlidesDetails.  # noqa: E501
        :type: int
        """

        self._ooxml_id = ooxml_id

    @property
    def svg_blob_url(self):
        """Gets the svg_blob_url of this SlideSlidesDetails.  # noqa: E501


        :return: The svg_blob_url of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._svg_blob_url

    @svg_blob_url.setter
    def svg_blob_url(self, svg_blob_url):
        """Sets the svg_blob_url of this SlideSlidesDetails.


        :param svg_blob_url: The svg_blob_url of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._svg_blob_url = svg_blob_url

    @property
    def slide_document_url(self):
        """Gets the slide_document_url of this SlideSlidesDetails.  # noqa: E501


        :return: The slide_document_url of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._slide_document_url

    @slide_document_url.setter
    def slide_document_url(self, slide_document_url):
        """Sets the slide_document_url of this SlideSlidesDetails.


        :param slide_document_url: The slide_document_url of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._slide_document_url = slide_document_url

    @property
    def theme(self):
        """Gets the theme of this SlideSlidesDetails.  # noqa: E501


        :return: The theme of this SlideSlidesDetails.  # noqa: E501
        :rtype: object
        """
        return self._theme

    @theme.setter
    def theme(self, theme):
        """Sets the theme of this SlideSlidesDetails.


        :param theme: The theme of this SlideSlidesDetails.  # noqa: E501
        :type: object
        """

        self._theme = theme

    @property
    def shape_tree(self):
        """Gets the shape_tree of this SlideSlidesDetails.  # noqa: E501


        :return: The shape_tree of this SlideSlidesDetails.  # noqa: E501
        :rtype: object
        """
        return self._shape_tree

    @shape_tree.setter
    def shape_tree(self, shape_tree):
        """Sets the shape_tree of this SlideSlidesDetails.


        :param shape_tree: The shape_tree of this SlideSlidesDetails.  # noqa: E501
        :type: object
        """

        self._shape_tree = shape_tree

    @property
    def base_element_blob_url(self):
        """Gets the base_element_blob_url of this SlideSlidesDetails.  # noqa: E501


        :return: The base_element_blob_url of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._base_element_blob_url

    @base_element_blob_url.setter
    def base_element_blob_url(self, base_element_blob_url):
        """Sets the base_element_blob_url of this SlideSlidesDetails.


        :param base_element_blob_url: The base_element_blob_url of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._base_element_blob_url = base_element_blob_url

    @property
    def changed_base_element_blob_url(self):
        """Gets the changed_base_element_blob_url of this SlideSlidesDetails.  # noqa: E501


        :return: The changed_base_element_blob_url of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._changed_base_element_blob_url

    @changed_base_element_blob_url.setter
    def changed_base_element_blob_url(self, changed_base_element_blob_url):
        """Sets the changed_base_element_blob_url of this SlideSlidesDetails.


        :param changed_base_element_blob_url: The changed_base_element_blob_url of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._changed_base_element_blob_url = changed_base_element_blob_url

    @property
    def package_uri(self):
        """Gets the package_uri of this SlideSlidesDetails.  # noqa: E501


        :return: The package_uri of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._package_uri

    @package_uri.setter
    def package_uri(self, package_uri):
        """Sets the package_uri of this SlideSlidesDetails.


        :param package_uri: The package_uri of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._package_uri = package_uri

    @property
    def name(self):
        """Gets the name of this SlideSlidesDetails.  # noqa: E501


        :return: The name of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SlideSlidesDetails.


        :param name: The name of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this SlideSlidesDetails.  # noqa: E501


        :return: The id of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this SlideSlidesDetails.


        :param id: The id of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def date_created(self):
        """Gets the date_created of this SlideSlidesDetails.  # noqa: E501


        :return: The date_created of this SlideSlidesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_created

    @date_created.setter
    def date_created(self, date_created):
        """Sets the date_created of this SlideSlidesDetails.


        :param date_created: The date_created of this SlideSlidesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_created = date_created

    @property
    def user_created(self):
        """Gets the user_created of this SlideSlidesDetails.  # noqa: E501


        :return: The user_created of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_created

    @user_created.setter
    def user_created(self, user_created):
        """Sets the user_created of this SlideSlidesDetails.


        :param user_created: The user_created of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._user_created = user_created

    @property
    def date_modified(self):
        """Gets the date_modified of this SlideSlidesDetails.  # noqa: E501


        :return: The date_modified of this SlideSlidesDetails.  # noqa: E501
        :rtype: datetime
        """
        return self._date_modified

    @date_modified.setter
    def date_modified(self, date_modified):
        """Sets the date_modified of this SlideSlidesDetails.


        :param date_modified: The date_modified of this SlideSlidesDetails.  # noqa: E501
        :type: datetime
        """

        self._date_modified = date_modified

    @property
    def user_modified(self):
        """Gets the user_modified of this SlideSlidesDetails.  # noqa: E501


        :return: The user_modified of this SlideSlidesDetails.  # noqa: E501
        :rtype: str
        """
        return self._user_modified

    @user_modified.setter
    def user_modified(self, user_modified):
        """Sets the user_modified of this SlideSlidesDetails.


        :param user_modified: The user_modified of this SlideSlidesDetails.  # noqa: E501
        :type: str
        """

        self._user_modified = user_modified

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SlideSlidesDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SlideSlidesDetails):
            return True

        return self.to_dict() != other.to_dict()
